@@ -22,8 +22,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#ifndef __GDK_EVENTS_PRIVATE_H__
-#define __GDK_EVENTS_PRIVATE_H__
+#pragma once
 
 #include <gdk/gdktypes.h>
 #include <gdk/gdkdrag.h>
@@ -206,12 +205,11 @@ struct _GdkTouchEvent
  *   %GDK_SCROLL_SMOOTH).
  * @delta_x: the x coordinate of the scroll delta
  * @delta_y: the y coordinate of the scroll delta
- * @pointer_emulated: whether the scroll event was the result of
- *   a pointer emulation
  * @tool: a `GdkDeviceTool`
  * @history: (element-type GdkTimeCoord): array of times and deltas
  *   for other scroll events that were compressed before delivering the
  *   current event
+ * @unit: The scroll unit in which delta_x and delta_y are represented.
  *
  * Generated from button presses for the buttons 4 to 7. Wheel mice are
  * usually configured to generate button press events for buttons 4 and 5
@@ -230,10 +228,10 @@ struct _GdkScrollEvent
   GdkScrollDirection direction;
   double delta_x;
   double delta_y;
-  gboolean pointer_emulated;
   gboolean is_stop;
   GdkDeviceTool *tool;
   GArray *history; /* <GdkTimeCoord> */
+  GdkScrollUnit unit;
 };
 
 /*
@@ -260,6 +258,9 @@ typedef struct {
  * @keycode: the raw code of the key that was pressed or released.
  * @translated: the result of translating @keycode. First with the full
  *   @state, then while ignoring Caps Lock.
+ * @compose_sequence: optional string for use by selected IM modules.
+ *   Contains either partial compose sequences or the final composed
+ *   string of the keystroke sequence.
  *
  * Describes a key press or key release event.
  */
@@ -271,6 +272,7 @@ struct _GdkKeyEvent
   guint32 keycode;
   gboolean key_is_modifier;
   GdkTranslatedKey translated[2];
+  char *compose_sequence;
 };
 
 /*
@@ -471,7 +473,8 @@ GdkEvent * gdk_key_event_new            (GdkEventType      type,
                                          GdkModifierType   modifiers,
                                          gboolean          is_modifier,
                                          GdkTranslatedKey *translated,
-                                         GdkTranslatedKey *no_lock);
+                                         GdkTranslatedKey *no_lock,
+                                         char             *compose_sequence);
 
 GdkEvent * gdk_focus_event_new          (GdkSurface      *surface,
                                          GdkDevice       *device,
@@ -486,15 +489,24 @@ GdkEvent * gdk_scroll_event_new         (GdkSurface      *surface,
                                          GdkModifierType  state,
                                          double           delta_x,
                                          double           delta_y,
-                                         gboolean         is_stop);
+                                         gboolean         is_stop,
+                                         GdkScrollUnit    unit);
 
 GdkEvent * gdk_scroll_event_new_discrete (GdkSurface         *surface,
+                                          GdkDevice          *device,
+                                          GdkDeviceTool      *tool,
+                                          guint32             time,
+                                          GdkModifierType     state,
+                                          GdkScrollDirection  direction);
+
+GdkEvent * gdk_scroll_event_new_value120 (GdkSurface         *surface,
                                           GdkDevice          *device,
                                           GdkDeviceTool      *tool,
                                           guint32             time,
                                           GdkModifierType     state,
                                           GdkScrollDirection  direction,
-                                          gboolean            emulated);
+                                          double              delta_x,
+                                          double              delta_y);
 
 GdkEvent * gdk_touch_event_new          (GdkEventType      type,
                                          GdkEventSequence *sequence,
@@ -534,6 +546,7 @@ GdkEvent * gdk_touchpad_event_new_pinch (GdkSurface              *surface,
                                          double                   angle_delta);
 
 GdkEvent * gdk_touchpad_event_new_hold  (GdkSurface              *surface,
+                                         GdkEventSequence        *sequence,
                                          GdkDevice               *device,
                                          guint32                  time,
                                          GdkModifierType          state,
@@ -588,6 +601,8 @@ GdkEvent * gdk_grab_broken_event_new    (GdkSurface      *surface,
 GdkTranslatedKey *      gdk_key_event_get_translated_key        (GdkEvent *event,
                                                                  gboolean  no_lock);
 
+char *                  gdk_key_event_get_compose_sequence      (GdkEvent *event);
+
 typedef enum
 {
   /* Following flag is set for events on the event queue during
@@ -617,7 +632,5 @@ void    _gdk_event_queue_flush                     (GdkDisplay       *display);
 
 double * gdk_event_dup_axes (GdkEvent *event);
 
-
 G_END_DECLS
 
-#endif /* __GDK_EVENTS_PRIVATE_H__ */
