@@ -27,10 +27,12 @@
 #import "GdkMacosView.h"
 #import "GdkMacosWindow.h"
 
-#include "gdkmacosclipboard-private.h"
 #include "gdkmacosdisplay-private.h"
+#include "gdkmacosdrag-private.h"
 #include "gdkmacosdrop-private.h"
+#include "gdkmacoseventsource-private.h"
 #include "gdkmacosmonitor-private.h"
+#include "gdkmacospasteboard-private.h"
 #include "gdkmacossurface-private.h"
 #include "gdkmacospopupsurface-private.h"
 #include "gdkmacostoplevelsurface-private.h"
@@ -71,7 +73,10 @@ typedef NSString *CALayerContentsGravity;
 
 -(void)windowDidMiniaturize:(NSNotification *)aNotification
 {
-  gdk_synthesize_surface_state (GDK_SURFACE (gdk_surface), 0, GDK_TOPLEVEL_STATE_MINIMIZED);
+  gdk_synthesize_surface_state (GDK_SURFACE (gdk_surface),
+                                0,
+                                GDK_TOPLEVEL_STATE_MINIMIZED |
+                                GDK_TOPLEVEL_STATE_SUSPENDED);
 }
 
 -(void)windowDidDeminiaturize:(NSNotification *)aNotification
@@ -81,7 +86,10 @@ typedef NSString *CALayerContentsGravity;
   else if (GDK_IS_MACOS_POPUP_SURFACE (gdk_surface))
     _gdk_macos_popup_surface_attach_to_parent (GDK_MACOS_POPUP_SURFACE (gdk_surface));
 
-  gdk_synthesize_surface_state (GDK_SURFACE (gdk_surface), GDK_TOPLEVEL_STATE_MINIMIZED, 0);
+  gdk_synthesize_surface_state (GDK_SURFACE (gdk_surface),
+                                GDK_TOPLEVEL_STATE_MINIMIZED |
+                                GDK_TOPLEVEL_STATE_SUSPENDED,
+                                0);
 }
 
 -(void)windowDidBecomeKey:(NSNotification *)aNotification
@@ -144,7 +152,7 @@ typedef NSString *CALayerContentsGravity;
        *
        * TODO: Can we improve grab breaking to fix this?
        */
-      _gdk_macos_display_send_button_event ([self gdkDisplay], event);
+      _gdk_macos_display_send_event ([self gdkDisplay], event);
 
       _gdk_macos_display_break_all_grabs (GDK_MACOS_DISPLAY (display), time);
 
@@ -246,7 +254,7 @@ typedef NSString *CALayerContentsGravity;
   [view release];
 
   /* TODO: We might want to make this more extensible at some point */
-  _gdk_macos_clipboard_register_drag_types (self);
+  _gdk_macos_pasteboard_register_drag_types (self);
 
   return self;
 }
@@ -576,6 +584,8 @@ typedef NSString *CALayerContentsGravity;
   initialResizeLocation = convert_nspoint_to_screen (self, [self mouseLocationOutsideOfEventStream]);
 }
 
+// NSDraggingDestination protocol
+
 -(NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
 {
   NSPoint location = [sender draggingLocation];
@@ -665,10 +675,64 @@ typedef NSString *CALayerContentsGravity;
   return NO;
 }
 
--(void)draggedImage:(NSImage *)anImage endedAt:(NSPoint)aPoint operation:(NSDragOperation)operation
+// NSDraggingSource protocol
+
+- (NSDragOperation)draggingSession:(NSDraggingSession *)session sourceOperationMaskForDraggingContext:(NSDraggingContext)context
 {
+  NSInteger sequence_number = [session draggingSequenceNumber];
+  GdkDisplay *display = gdk_surface_get_display (GDK_SURFACE (gdk_surface));
+  GdkDrag *drag = _gdk_macos_display_find_drag (GDK_MACOS_DISPLAY (display), sequence_number);
+  GdkModifierType state = _gdk_macos_display_get_current_keyboard_modifiers (GDK_MACOS_DISPLAY (display));
+
+  _gdk_macos_drag_set_actions (GDK_MACOS_DRAG (drag), state);
+
+  return _gdk_macos_drag_operation (GDK_MACOS_DRAG (drag));
 }
 
+- (void)draggingSession:(NSDraggingSession *)session willBeginAtPoint:(NSPoint)screenPoint
+{
+  NSInteger sequence_number = [session draggingSequenceNumber];
+  GdkDisplay *display = gdk_surface_get_display (GDK_SURFACE (gdk_surface));
+  GdkDrag *drag = _gdk_macos_display_find_drag (GDK_MACOS_DISPLAY (display), sequence_number);
+  int x, y;
+
+  _gdk_macos_display_from_display_coords (GDK_MACOS_DISPLAY (display), screenPoint.x, screenPoint.y, &x, &y);
+  _gdk_macos_drag_set_start_position (GDK_MACOS_DRAG (drag), x, y);
+  _gdk_macos_drag_surface_move (GDK_MACOS_DRAG (drag), x, y);
+}
+
+- (void)draggingSession:(NSDraggingSession *)session movedToPoint:(NSPoint)screenPoint
+{
+  NSInteger sequence_number = [session draggingSequenceNumber];
+  GdkMacosDisplay *display = GDK_MACOS_DISPLAY (gdk_surface_get_display (GDK_SURFACE (gdk_surface)));
+  GdkDrag *drag = _gdk_macos_display_find_drag (GDK_MACOS_DISPLAY (display), sequence_number);
+  int x, y;
+
+  _gdk_macos_display_send_event (display, [NSApp currentEvent]);
+
+  _gdk_macos_display_from_display_coords (display, screenPoint.x, screenPoint.y, &x, &y);
+  _gdk_macos_drag_surface_move (GDK_MACOS_DRAG (drag), x, y);
+}
+
+- (void)draggingSession:(NSDraggingSession *)session endedAtPoint:(NSPoint)screenPoint operation:(NSDragOperation)operation
+{
+  NSInteger sequence_number = [session draggingSequenceNumber];
+  GdkMacosDisplay *display = GDK_MACOS_DISPLAY (gdk_surface_get_display (GDK_SURFACE (gdk_surface)));
+  GdkDrag *drag = _gdk_macos_display_find_drag (display, sequence_number);
+
+  _gdk_macos_display_send_event (display, [NSApp currentEvent]);
+  gdk_drag_set_selected_action (drag, _gdk_macos_drag_ns_operation_to_action (operation));
+
+  if (gdk_drag_get_selected_action (drag) != 0)
+    g_signal_emit_by_name (drag, "drop-performed");
+  else
+    gdk_drag_cancel (drag, GDK_DRAG_CANCEL_NO_TARGET);
+
+  _gdk_macos_display_set_drag (display, [session draggingSequenceNumber], NULL);
+}
+
+// end
+
 -(void)setStyleMask:(NSWindowStyleMask)styleMask
 {
   gboolean was_fullscreen;
