@@ -1,6 +1,7 @@
 /* GDK - The GIMP Drawing Kit
  * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
  * Copyright (C) 1998-2002 Tor Lillqvist
+ * Copyright (C) 2023 the GTK team
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,13 +17,6 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/*
- * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
- * file for a list of people on the GTK+ Team.  See the ChangeLog
- * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
- */
-
 #include "config.h"
 
 #include <glib/gprintf.h>
@@ -32,10 +26,11 @@
 #include <io.h>
 
 #include "gdk.h"
-#include "gdkdebug.h"
+#include "gdkdebugprivate.h"
 #include "gdkkeysyms.h"
-#include "gdkintl.h"
+#include <glib/gi18n-lib.h>
 #include "gdkprivate-win32.h"
+#include "gdkinput-dmanipulation.h"
 #include "gdkwin32.h"
 
 #include <objbase.h>
@@ -44,29 +39,72 @@
 #include <wintab.h>
 #include <imm.h>
 
-/* for CFSTR_SHELLIDLIST */
-#include <shlobj.h>
+/* Whether GDK initialized COM */
+static gboolean co_initialized = FALSE;
 
-static gboolean gdk_synchronize = FALSE;
+/* Whether GDK initialized OLE */
+static gboolean ole_initialized = FALSE;
 
 void
 _gdk_win32_surfaceing_init (void)
 {
-  GdkWin32Keymap *win32_keymap;
-
-  win32_keymap = GDK_WIN32_KEYMAP (_gdk_win32_display_get_keymap (_gdk_display));
-
-  if (gdk_synchronize)
-    GdiSetBatchLimit (1);
-
-  _gdk_app_hmodule = GetModuleHandle (NULL);
-  _gdk_display_hdc = CreateDC ("DISPLAY", NULL, NULL, NULL);
-  _gdk_input_locale = GetKeyboardLayout (0);
-  _gdk_win32_keymap_set_active_layout (win32_keymap, _gdk_input_locale);
-
-  GDK_NOTE (EVENTS, g_print ("input_locale: %p\n", _gdk_input_locale));
-
   _gdk_win32_clipdrop_init ();
+
+  gdk_dmanipulation_initialize ();
+}
+
+gboolean
+gdk_win32_ensure_com (void)
+{
+  if (!co_initialized)
+    {
+      /* UI thread should only use STA model. See
+       * -> https://devblogs.microsoft.com/oldnewthing/20080424-00/?p=22603
+       * -> https://devblogs.microsoft.com/oldnewthing/20071018-00/?p=24743
+       */
+      const DWORD flags = COINIT_APARTMENTTHREADED |
+                          COINIT_DISABLE_OLE1DDE;
+      HRESULT hr;
+
+      hr = CoInitializeEx (NULL, flags);
+      if (SUCCEEDED (hr))
+        co_initialized = TRUE;
+      else switch (hr)
+        {
+        case RPC_E_CHANGED_MODE:
+          g_warning ("COM runtime already initialized on the main "
+                     "thread with an incompatible apartment model");
+        break;
+        default:
+          HR_LOG (hr);
+        break;
+        }
+    }
+
+  return co_initialized;
+}
+
+gboolean
+gdk_win32_ensure_ole (void)
+{
+  if (!ole_initialized)
+    {
+      HRESULT hr = OleInitialize (NULL);
+      if (SUCCEEDED (hr))
+        ole_initialized = TRUE;
+      else switch (hr)
+        {
+        case RPC_E_CHANGED_MODE:
+          g_warning ("Failed to initialize the OLE2 runtime because "
+                     "the thread has an incompatible apartment model");
+        break;
+        default:
+          HR_LOG (hr);
+        break;
+        }
+    }
+
+  return ole_initialized;
 }
 
 void
@@ -124,135 +162,6 @@ static_printf (const char *format,
   return retval;
 }
 
-void
-_gdk_win32_print_paletteentries (const PALETTEENTRY *pep,
-				const int           nentries)
-{
-  char buf[20];
-  int i;
-
-  for (i = 0; i < nentries; i++)
-    g_print ("  %3d %02x:  %02x %02x %02x%s\n",
-	     i, i,
-	     pep[i].peRed, pep[i].peGreen, pep[i].peBlue,
-	     (pep[i].peFlags == 0 ? "" :
-	      (pep[i].peFlags == PC_EXPLICIT ? " PC_EXPLICIT" :
-	       (pep[i].peFlags == PC_NOCOLLAPSE ? " PC_NOCOLLAPSE" :
-		(pep[i].peFlags == PC_RESERVED ? " PC_RESERVED" :
-		 (g_sprintf (buf, " %d", pep[i].peFlags), buf))))));
-}
-
-void
-_gdk_win32_print_system_palette (void)
-{
-  PALETTEENTRY *pe;
-  int k;
-
-  k = GetSystemPaletteEntries (_gdk_display_hdc, 0, 0, NULL);
-  pe = g_new (PALETTEENTRY, k);
-  k = GetSystemPaletteEntries (_gdk_display_hdc, 0, k, pe);
-
-  if (!k)
-    g_print ("GetSystemPaletteEntries failed: %s\n",
-	     g_win32_error_message (GetLastError ()));
-  else
-    {
-      g_print ("System palette: %d entries\n", k);
-      _gdk_win32_print_paletteentries (pe, k);
-    }
-  g_free (pe);
-}
-
-static int
-palette_size (HPALETTE hpal)
-{
-  WORD npal = 0;
-
-  if (!GetObject (hpal, sizeof (npal), &npal))
-    WIN32_GDI_FAILED ("GetObject (HPALETTE)");
-
-  return npal;
-}
-
-void
-_gdk_win32_print_hpalette (HPALETTE hpal)
-{
-  PALETTEENTRY *pe;
-  int n, npal;
-
-  npal = palette_size (hpal);
-  pe = g_new (PALETTEENTRY, npal);
-  n = GetPaletteEntries (hpal, 0, npal, pe);
-
-  if (!n)
-    g_print ("HPALETTE %p: GetPaletteEntries failed: %s\n",
-	     hpal, g_win32_error_message (GetLastError ()));
-  else
-    {
-      g_print ("HPALETTE %p: %d (%d) entries\n", hpal, n, npal);
-      _gdk_win32_print_paletteentries (pe, n);
-    }
-  g_free (pe);
-}
-
-void
-_gdk_win32_print_dc (HDC hdc)
-{
-  HGDIOBJ obj;
-  LOGBRUSH logbrush;
-  EXTLOGPEN extlogpen;
-  HRGN hrgn;
-  RECT rect;
-  int flag;
-
-  g_print ("%p:\n", hdc);
-  obj = GetCurrentObject (hdc, OBJ_BRUSH);
-  GetObject (obj, sizeof (LOGBRUSH), &logbrush);
-  g_print ("brush: %s color=%06lx hatch=%p\n",
-	   _gdk_win32_lbstyle_to_string (logbrush.lbStyle),
-	   logbrush.lbColor, (gpointer) logbrush.lbHatch);
-  obj = GetCurrentObject (hdc, OBJ_PEN);
-  GetObject (obj, sizeof (EXTLOGPEN), &extlogpen);
-  g_print ("pen: %s %s %s %s w=%d %s\n",
-	   _gdk_win32_pstype_to_string (extlogpen.elpPenStyle),
-	   _gdk_win32_psstyle_to_string (extlogpen.elpPenStyle),
-	   _gdk_win32_psendcap_to_string (extlogpen.elpPenStyle),
-	   _gdk_win32_psjoin_to_string (extlogpen.elpPenStyle),
-	   (int) extlogpen.elpWidth,
-	   _gdk_win32_lbstyle_to_string (extlogpen.elpBrushStyle));
-  g_print ("rop2: %s textcolor=%06lx\n",
-	   _gdk_win32_rop2_to_string (GetROP2 (hdc)),
-	   GetTextColor (hdc));
-  hrgn = CreateRectRgn (0, 0, 0, 0);
-  if ((flag = GetClipRgn (hdc, hrgn)) == -1)
-    WIN32_API_FAILED ("GetClipRgn");
-  else if (flag == 0)
-    g_print ("no clip region\n");
-  else if (flag == 1)
-    {
-      GetRgnBox (hrgn, &rect);
-      g_print ("clip region: %p bbox: %s\n",
-	       hrgn, _gdk_win32_rect_to_string (&rect));
-    }
-  DeleteObject (hrgn);
-}
-
-char *
-_gdk_win32_drag_protocol_to_string (GdkDragProtocol protocol)
-{
-  switch (protocol)
-    {
-#define CASE(x) case GDK_DRAG_PROTO_##x: return #x
-      CASE (NONE);
-      CASE (WIN32_DROPFILES);
-      CASE (OLE2);
-#undef CASE
-    default: return static_printf ("illegal_%d", protocol);
-    }
-  /* NOTREACHED */
-  return NULL;
-}
-
 char *
 _gdk_win32_surface_state_to_string (GdkToplevelState state)
 {
@@ -415,7 +324,7 @@ _gdk_win32_drag_action_to_string (GdkDragAction actions)
   return static_printf ("%s", buf);
 }
 
-char *
+static char *
 _gdk_win32_rop2_to_string (int rop2)
 {
   switch (rop2)
@@ -444,7 +353,7 @@ _gdk_win32_rop2_to_string (int rop2)
   return NULL;
 }
 
-char *
+static char *
 _gdk_win32_lbstyle_to_string (UINT brush_style)
 {
   switch (brush_style)
@@ -463,7 +372,7 @@ _gdk_win32_lbstyle_to_string (UINT brush_style)
   return NULL;
 }
 
-char *
+static char *
 _gdk_win32_pstype_to_string (DWORD pen_style)
 {
   switch (pen_style & PS_TYPE_MASK)
@@ -476,7 +385,7 @@ _gdk_win32_pstype_to_string (DWORD pen_style)
   return NULL;
 }
 
-char *
+static char *
 _gdk_win32_psstyle_to_string (DWORD pen_style)
 {
   switch (pen_style & PS_STYLE_MASK)
@@ -498,7 +407,7 @@ _gdk_win32_psstyle_to_string (DWORD pen_style)
   return NULL;
 }
 
-char *
+static char *
 _gdk_win32_psendcap_to_string (DWORD pen_style)
 {
   switch (pen_style & PS_ENDCAP_MASK)
@@ -514,7 +423,7 @@ _gdk_win32_psendcap_to_string (DWORD pen_style)
   return NULL;
 }
 
-char *
+static char *
 _gdk_win32_psjoin_to_string (DWORD pen_style)
 {
   switch (pen_style & PS_JOIN_MASK)
@@ -530,6 +439,56 @@ _gdk_win32_psjoin_to_string (DWORD pen_style)
   return NULL;
 }
 
+void
+_gdk_win32_print_dc (HDC hdc)
+{
+  HGDIOBJ obj;
+  LOGBRUSH logbrush;
+  EXTLOGPEN extlogpen;
+  HRGN hrgn;
+  RECT rect;
+  int flag;
+
+  g_print ("%p:\n", hdc);
+
+  obj = GetCurrentObject (hdc, OBJ_BRUSH);
+  GetObject (obj, sizeof (LOGBRUSH), &logbrush);
+
+  g_print ("brush: %s color=%06lx hatch=%p\n",
+           _gdk_win32_lbstyle_to_string (logbrush.lbStyle),
+           logbrush.lbColor, (gpointer) logbrush.lbHatch);
+
+  obj = GetCurrentObject (hdc, OBJ_PEN);
+  GetObject (obj, sizeof (EXTLOGPEN), &extlogpen);
+
+  g_print ("pen: %s %s %s %s w=%d %s\n",
+           _gdk_win32_pstype_to_string (extlogpen.elpPenStyle),
+           _gdk_win32_psstyle_to_string (extlogpen.elpPenStyle),
+           _gdk_win32_psendcap_to_string (extlogpen.elpPenStyle),
+           _gdk_win32_psjoin_to_string (extlogpen.elpPenStyle),
+           (int) extlogpen.elpWidth,
+           _gdk_win32_lbstyle_to_string (extlogpen.elpBrushStyle));
+
+  g_print ("rop2: %s textcolor=%06lx\n",
+           _gdk_win32_rop2_to_string (GetROP2 (hdc)),
+           GetTextColor (hdc));
+
+  hrgn = CreateRectRgn (0, 0, 0, 0);
+
+  if ((flag = GetClipRgn (hdc, hrgn)) == -1)
+    WIN32_API_FAILED ("GetClipRgn");
+  else if (flag == 0)
+    g_print ("no clip region\n");
+  else if (flag == 1)
+    {
+      GetRgnBox (hrgn, &rect);
+      g_print ("clip region: %p bbox: %s\n",
+               hrgn, _gdk_win32_rect_to_string (&rect));
+    }
+
+  DeleteObject (hrgn);
+}
+
 char *
 _gdk_win32_message_to_string (UINT msg)
 {
@@ -830,26 +789,6 @@ _gdk_win32_cf_to_string (UINT format)
     }
 }
 
-char *
-_gdk_win32_data_to_string (const guchar *data,
-			   int           nbytes)
-{
-  GString *s = g_string_new ("");
-  int i;
-  char *retval;
-
-  for (i = 0; i < nbytes; i++)
-    if (data[i] >=' ' && data[i] <= '~')
-      g_string_append_printf (s, "%c  ", data[i]);
-    else
-      g_string_append_printf (s, "%02X ", data[i]);
-
-  retval = static_printf ("%s", s->str);
-  g_string_free (s, TRUE);
-
-  return retval;
-}
-
 char *
 _gdk_win32_rect_to_string (const RECT *rect)
 {
@@ -858,34 +797,4 @@ _gdk_win32_rect_to_string (const RECT *rect)
 			rect->left, rect->top);
 }
 
-char *
-_gdk_win32_gdkrectangle_to_string (const GdkRectangle *rect)
-{
-  return static_printf ("%dx%d@%+d%+d",
-			rect->width, rect->height,
-			rect->x, rect->y);
-}
-
-char *
-_gdk_win32_cairo_region_to_string (const cairo_region_t *rgn)
-{
-  cairo_rectangle_int_t extents;
-  cairo_region_get_extents (rgn, &extents);
-  return static_printf ("%dx%d@%+d%+d",
-			extents.width, extents.height,
-			extents.x, extents.y);
-}
-
-char *
-_gdk_win32_surface_description (GdkSurface *d)
-{
-  g_return_val_if_fail (GDK_IS_SURFACE (d), NULL);
-
-  return static_printf ("%s:%p:%dx%d",
-			G_OBJECT_TYPE_NAME (d),
-			GDK_SURFACE_HWND (d),
-			gdk_surface_get_width (GDK_SURFACE (d)),
-                        gdk_surface_get_height (GDK_SURFACE (d)));
-}
-
 #endif /* G_ENABLE_DEBUG */
