@@ -314,6 +314,8 @@ handle_accessible_method (GDBusConnection       *connection,
       const char *path = gtk_at_spi_context_get_context_path (GTK_AT_SPI_CONTEXT (context));
 
       g_dbus_method_invocation_return_value (invocation, g_variant_new ("((so))", name, path));
+
+      g_object_unref (context);
     }
   else if (g_strcmp0 (method_name, "GetChildren") == 0)
     {
@@ -334,6 +336,8 @@ handle_accessible_method (GDBusConnection       *connection,
           const char *path = gtk_at_spi_context_get_context_path (GTK_AT_SPI_CONTEXT (context));
 
           g_variant_builder_add (&builder, "(so)", name, path);
+
+          g_object_unref (context);
         }
 
       g_dbus_method_invocation_return_value (invocation, g_variant_new ("(a(so))", &builder));
@@ -453,6 +457,8 @@ gtk_at_spi_root_child_changed (GtkAtSpiRoot             *self,
       GtkATContext *context = gtk_accessible_get_at_context (child);
 
       window_ref = gtk_at_spi_context_to_ref (GTK_AT_SPI_CONTEXT (context));
+
+      g_object_unref (context);
     }
 
   switch (change)
@@ -507,9 +513,9 @@ on_registration_reply (GObject      *gobject,
                      &self->desktop_path);
       g_variant_unref (reply);
 
-      GTK_NOTE (A11Y, g_message ("Connected to the a11y registry at (%s, %s)",
-                                 self->desktop_name,
-                                 self->desktop_path));
+      GTK_DEBUG (A11Y, "Connected to the a11y registry at (%s, %s)",
+                       self->desktop_name,
+                       self->desktop_path);
     }
 
   /* Register the cache object */
@@ -581,9 +587,9 @@ root_register (gpointer user_data)
                                      NULL,
                                      NULL);
 
-  GTK_NOTE (A11Y, g_message ("Registering (%s, %s) on the a11y bus",
-                             unique_name,
-                             self->root_path));
+  GTK_DEBUG (A11Y, "Registering (%s, %s) on the a11y bus",
+                   unique_name,
+                   self->root_path);
 
   g_dbus_connection_call (self->connection,
                           "org.a11y.atspi.Registry",
@@ -690,7 +696,8 @@ gtk_at_spi_root_constructed (GObject *gobject)
       /* No need to validate the path */
       self->base_path = g_strconcat (app_path, "/a11y", NULL);
     }
-  else
+
+  if (self->base_path == NULL)
     {
       const char *program_name = g_get_prgname ();
 
