@@ -61,7 +61,7 @@ gtk_css_parser_new (GtkCssTokenizer       *tokenizer,
 {
   GtkCssParser *self;
 
-  self = g_slice_new0 (GtkCssParser);
+  self = g_new0 (GtkCssParser, 1);
 
   self->ref_count = 1;
   self->tokenizer = gtk_css_tokenizer_ref (tokenizer);
@@ -130,7 +130,7 @@ gtk_css_parser_finalize (GtkCssParser *self)
     g_critical ("Finalizing CSS parser with %u remaining blocks", self->blocks->len);
   g_array_free (self->blocks, TRUE);
 
-  g_slice_free (GtkCssParser, self);
+  g_free (self);
 }
 
 GtkCssParser *
@@ -642,13 +642,13 @@ gtk_css_parser_consume_function (GtkCssParser *self,
 {
   const GtkCssToken *token;
   gboolean result = FALSE;
-  char *function_name;
+  char function_name[64];
   guint arg;
 
   token = gtk_css_parser_get_token (self);
   g_return_val_if_fail (gtk_css_token_is (token, GTK_CSS_TOKEN_FUNCTION), FALSE);
 
-  function_name = g_strdup (token->string.string);
+  g_strlcpy (function_name, gtk_css_token_get_string (token), 64);
   gtk_css_parser_start_block (self);
 
   arg = 0;
@@ -691,7 +691,6 @@ gtk_css_parser_consume_function (GtkCssParser *self,
     }
 
   gtk_css_parser_end_block (self);
-  g_free (function_name);
 
   return result;
 }
@@ -734,7 +733,7 @@ gtk_css_parser_has_ident (GtkCssParser *self,
   token = gtk_css_parser_get_token (self);
 
   return gtk_css_token_is (token, GTK_CSS_TOKEN_IDENT) &&
-         g_ascii_strcasecmp (token->string.string, ident) == 0;
+         g_ascii_strcasecmp (gtk_css_token_get_string (token), ident) == 0;
 }
 
 gboolean
@@ -756,7 +755,7 @@ gtk_css_parser_has_integer (GtkCssParser *self)
  * Checks if the next token is a function with the given @name.
  *
  * Returns: %TRUE if the next token is a function with the given @name
- **/
+ */
 gboolean
 gtk_css_parser_has_function (GtkCssParser *self,
                              const char   *name)
@@ -766,7 +765,7 @@ gtk_css_parser_has_function (GtkCssParser *self,
   token = gtk_css_parser_get_token (self);
 
   return gtk_css_token_is (token, GTK_CSS_TOKEN_FUNCTION) &&
-         g_ascii_strcasecmp (token->string.string, name) == 0;
+         g_ascii_strcasecmp (gtk_css_token_get_string (token), name) == 0;
 }
 
 /**
@@ -819,7 +818,7 @@ gtk_css_parser_try_ident (GtkCssParser *self,
   token = gtk_css_parser_get_token (self);
 
   if (!gtk_css_token_is (token, GTK_CSS_TOKEN_IDENT) ||
-      g_ascii_strcasecmp (token->string.string, ident) != 0)
+      g_ascii_strcasecmp (gtk_css_token_get_string (token), ident) != 0)
     return FALSE;
 
   gtk_css_parser_consume_token (self);
@@ -846,7 +845,7 @@ gtk_css_parser_try_at_keyword (GtkCssParser *self,
   token = gtk_css_parser_get_token (self);
 
   if (!gtk_css_token_is (token, GTK_CSS_TOKEN_AT_KEYWORD) ||
-      g_ascii_strcasecmp (token->string.string, keyword) != 0)
+      g_ascii_strcasecmp (gtk_css_token_get_string (token), keyword) != 0)
     return FALSE;
 
   gtk_css_parser_consume_token (self);
@@ -908,7 +907,7 @@ gtk_css_parser_consume_ident (GtkCssParser *self)
       return NULL;
     }
 
-  ident = g_strdup (token->string.string);
+  ident = g_strdup (gtk_css_token_get_string (token));
   gtk_css_parser_consume_token (self);
 
   return ident;
@@ -939,7 +938,7 @@ gtk_css_parser_consume_string (GtkCssParser *self)
       return NULL;
     }
 
-  ident = g_strdup (token->string.string);
+  ident = g_strdup (gtk_css_token_get_string (token));
   gtk_css_parser_consume_token (self);
 
   return ident;
@@ -963,7 +962,7 @@ gtk_css_parser_parse_url_arg (GtkCssParser *parser,
  * gtk_css_parser_consume_url:
  * @self: a `GtkCssParser`
  *
- * If the parser matches the <url> token from the [CSS
+ * If the parser matches the `<url>` token from the [CSS
  * specification](https://drafts.csswg.org/css-values-4/#url-value),
  * consumes it, resolves the URL and returns the resulting `GFile`.
  * On failure, an error is emitted and %NULL is returned.
@@ -980,7 +979,7 @@ gtk_css_parser_consume_url (GtkCssParser *self)
 
   if (gtk_css_token_is (token, GTK_CSS_TOKEN_URL))
     {
-      url = g_strdup (token->string.string);
+      url = g_strdup (gtk_css_token_get_string (token));
       gtk_css_parser_consume_token (self);
     }
   else if (gtk_css_token_is_function (token, "url"))
