@@ -25,11 +25,8 @@
 #include "gtkbox.h"
 #include "gtkbuildable.h"
 #include "gtkcenterbox.h"
-#include "gtkintl.h"
 #include "gtklabel.h"
-#include "gtknative.h"
 #include "gtkprivate.h"
-#include "gtksizerequest.h"
 #include "gtktypebuiltins.h"
 #include "gtkwidgetprivate.h"
 #include "gtkwindowcontrols.h"
@@ -62,7 +59,7 @@
  *
  * The `GtkHeaderBar` implementation of the `GtkBuildable` interface supports
  * adding children at the start or end sides by specifying “start” or “end” as
- * the “type” attribute of a <child> element, or setting the title widget by
+ * the “type” attribute of a `<child>` element, or setting the title widget by
  * specifying “title” value.
  *
  * By default the `GtkHeaderBar` uses a `GtkLabel` displaying the title of the
@@ -303,8 +300,7 @@ gtk_header_bar_set_title_widget (GtkHeaderBar *bar,
                                  GtkWidget    *title_widget)
 {
   g_return_if_fail (GTK_IS_HEADER_BAR (bar));
-  if (title_widget)
-    g_return_if_fail (GTK_IS_WIDGET (title_widget));
+  g_return_if_fail (title_widget == NULL || bar->title_widget == title_widget || gtk_widget_get_parent (title_widget) == NULL);
 
   /* No need to do anything if the title widget stays the same */
   if (bar->title_widget == title_widget)
@@ -567,7 +563,7 @@ gtk_header_bar_class_init (GtkHeaderBarClass *class)
   header_bar_props[PROP_TITLE_WIDGET] =
       g_param_spec_object ("title-widget", NULL, NULL,
                            GTK_TYPE_WIDGET,
-                           G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS);
+                           G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * GtkHeaderBar:show-title-buttons: (attributes org.gtk.Property.get=gtk_header_bar_get_show_title_buttons org.gtk.Property.set=gtk_header_bar_set_show_title_buttons)
@@ -778,7 +774,7 @@ gtk_header_bar_set_show_title_buttons (GtkHeaderBar *bar,
  * from those on the right. Recognized button names are minimize,
  * maximize, close and icon (the window icon).
  *
- * For example, “icon:minimize,maximize,close” specifies a icon
+ * For example, “icon:minimize,maximize,close” specifies an icon
  * on the left, and minimize, maximize and close buttons on the right.
  */
 void
