@@ -300,6 +300,8 @@ gtk_snapshot_new (void)
  * Returns the node that was constructed by @snapshot
  * and frees @snapshot.
  *
+ * See also [method@Gtk.Snapshot.to_node].
+ *
  * Returns: (transfer full) (nullable): a newly-created [class@Gsk.RenderNode]
  */
 GskRenderNode *
@@ -556,29 +558,29 @@ merge_color_matrix_nodes (const graphene_matrix_t *matrix2,
                           const graphene_vec4_t   *offset2,
                           GskRenderNode           *child)
 {
-  const graphene_matrix_t *mat1 = gsk_color_matrix_node_get_color_matrix (child);
+  const graphene_matrix_t *matrix1 = gsk_color_matrix_node_get_color_matrix (child);
   const graphene_vec4_t *offset1 = gsk_color_matrix_node_get_color_offset (child);
-  graphene_matrix_t mat2 = *matrix2;
-  graphene_vec4_t off2 = *offset2;
+  graphene_matrix_t matrix;
+  graphene_vec4_t offset;
   GskRenderNode *result;
 
   g_assert (gsk_render_node_get_node_type (child) == GSK_COLOR_MATRIX_NODE);
 
-  /* color matrix node: color = mat * p + offset; for a pixel p.
-   * color =  mat1 * (mat2 * p + offset2) + offset1;
-   *       =  mat1 * mat2  * p + offset2 * mat1 + offset1
-   *       = (mat1 * mat2) * p + (offset2 * mat1 + offset1)
+  /* color matrix node: color = trans(mat) * p + offset; for a pixel p.
+   * color =  trans(mat2) * (trans(mat1) * p + offset1) + offset2
+   *       =  trans(mat2) * trans(mat1) * p + trans(mat2) * offset1 + offset2
+   *       = trans(mat1 * mat2) * p + (trans(mat2) * offset1 + offset2)
    * Which this code does.
    * mat1 and offset1 come from @child.
    */
 
-  graphene_matrix_transform_vec4 (mat1, offset2, &off2);
-  graphene_vec4_add (&off2, offset1, &off2);
+  graphene_matrix_transform_vec4 (matrix2, offset1, &offset);
+  graphene_vec4_add (&offset, offset2, &offset);
 
-  graphene_matrix_multiply (mat1, &mat2, &mat2);
+  graphene_matrix_multiply (matrix1, matrix2, &matrix);
 
   result = gsk_color_matrix_node_new (gsk_color_matrix_node_get_child (child),
-                                      &mat2, &off2);
+                                      &matrix, &offset);
 
   return result;
 }
@@ -645,6 +647,13 @@ gtk_snapshot_collect_color_matrix (GtkSnapshot      *snapshot,
  * Modifies the colors of an image by applying an affine transformation
  * in RGB space.
  *
+ * In particular, the colors will be transformed by applying
+ *
+ *     pixel = transpose(color_matrix) * pixel + color_offset
+ *
+ * for every pixel. The transformation operates on unpremultiplied
+ * colors, with color components ordered R, G, B, A.
+ *
  * The image is recorded until the next call to [method@Gtk.Snapshot.pop].
  */
 void
@@ -699,6 +708,16 @@ gtk_snapshot_collect_repeat (GtkSnapshot      *snapshot,
   return repeat_node;
 }
 
+static GskRenderNode *
+gtk_snapshot_collect_discard_repeat (GtkSnapshot      *snapshot,
+                                     GtkSnapshotState *state,
+                                     GskRenderNode   **nodes,
+                                     guint             n_nodes)
+{
+  /* Drop the node and return nothing.  */
+  return NULL;
+}
+
 static void
 gtk_graphene_rect_scale_affine (const graphene_rect_t *rect,
                                 float                  scale_x,
@@ -789,17 +808,24 @@ gtk_snapshot_push_repeat (GtkSnapshot           *snapshot,
                           const graphene_rect_t *child_bounds)
 {
   GtkSnapshotState *state;
+  gboolean empty_child_bounds = FALSE;
   graphene_rect_t real_child_bounds = { { 0 } };
   float scale_x, scale_y, dx, dy;
 
   gtk_snapshot_ensure_affine (snapshot, &scale_x, &scale_y, &dx, &dy);
 
   if (child_bounds)
-    gtk_graphene_rect_scale_affine (child_bounds, scale_x, scale_y, dx, dy, &real_child_bounds);
+    {
+      gtk_graphene_rect_scale_affine (child_bounds, scale_x, scale_y, dx, dy, &real_child_bounds);
+      if (real_child_bounds.size.width <= 0 || real_child_bounds.size.height <= 0)
+        empty_child_bounds = TRUE;
+    }
 
   state = gtk_snapshot_push_state (snapshot,
                                    gtk_snapshot_get_current_state (snapshot)->transform,
-                                   gtk_snapshot_collect_repeat,
+                                   empty_child_bounds
+                                   ? gtk_snapshot_collect_discard_repeat
+                                   : gtk_snapshot_collect_repeat,
                                    NULL);
 
   gtk_graphene_rect_scale_affine (bounds, scale_x, scale_y, dx, dy, &state->data.repeat.bounds);
@@ -1569,11 +1595,16 @@ gtk_snapshot_pop_collect (GtkSnapshot *snapshot)
  * Returns the render node that was constructed
  * by @snapshot.
  *
+ * Note that this function may return %NULL if nothing has been
+ * added to the snapshot or if its content does not produce pixels
+ * to be rendered.
+ *
  * After calling this function, it is no longer possible to
  * add more nodes to @snapshot. The only function that should
  * be called after this is [method@GObject.Object.unref].
  *
- * Returns: (transfer full) (nullable): the constructed `GskRenderNode`
+ * Returns: (transfer full) (nullable): the constructed `GskRenderNode` or
+ *   %NULL if there are no nodes to render.
  */
 GskRenderNode *
 gtk_snapshot_to_node (GtkSnapshot *snapshot)
@@ -1827,9 +1858,10 @@ gtk_snapshot_translate_3d (GtkSnapshot              *snapshot,
  * @angle: the rotation angle, in degrees (clockwise)
  *
  * Rotates @@snapshot's coordinate system by @angle degrees in 2D space -
- * or in 3D speak, rotates around the Z axis.
+ * or in 3D speak, rotates around the Z axis. The rotation happens around
+ * the origin point of (0, 0) in the @snapshot's current coordinate system.
  *
- * To rotate around other axes, use [method@Gsk.Transform.rotate_3d].
+ * To rotate around axes other than the Z axis, use [method@Gsk.Transform.rotate_3d].
  */
 void
 gtk_snapshot_rotate (GtkSnapshot *snapshot,
