@@ -1,5 +1,7 @@
 #include <gtk/gtk.h>
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 #define FILE_INFO_TYPE_SELECTION (file_info_selection_get_type ())
 
 G_DECLARE_FINAL_TYPE (FileInfoSelection, file_info_selection, FILE_INFO, SELECTION, GObject)
@@ -477,7 +479,7 @@ row_data_free (gpointer _data)
 
   row_data_unbind (data);
 
-  g_slice_free (RowData, data);
+  g_free (data);
 }
 
 static void
@@ -487,7 +489,7 @@ setup_widget (GtkSignalListItemFactory *factory,
   GtkWidget *box, *child;
   RowData *data;
 
-  data = g_slice_new0 (RowData);
+  data = g_new0 (RowData, 1);
 
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
   gtk_list_item_set_child (list_item, box);
@@ -647,7 +649,7 @@ main (int argc, char *argv[])
 
   g_object_unref (selectionmodel);
 
-  gtk_widget_show (win);
+  gtk_window_present (GTK_WINDOW (win));
 
   toplevels = gtk_window_get_toplevels ();
   while (g_list_model_get_n_items (toplevels))
